==> kubernetes-client-1.0.0-sources.jar: io/fabric8/kubernetes/client/internal/BaseResourceList.java <==
/**
 * Copyright (C) 2015 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.fabric8.kubernetes.client.internal;

import com.ning.http.client.AsyncHttpClient;
import com.ning.http.client.Response;
import com.ning.http.client.ws.DefaultWebSocketListener;
import com.ning.http.client.ws.WebSocket;
import com.ning.http.client.ws.WebSocketUpgradeHandler;
import io.fabric8.kubernetes.api.builder.Builder;
import io.fabric8.kubernetes.api.model.Doneable;
import io.fabric8.kubernetes.api.model.HasMetadata;
import io.fabric8.kubernetes.api.model.KubernetesResourceList;
import io.fabric8.kubernetes.api.model.Status;
import io.fabric8.kubernetes.api.model.WatchEvent;
import io.fabric8.kubernetes.client.KubernetesClientException;
import io.fabric8.kubernetes.client.Watcher;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

public abstract class BaseResourceList<T extends HasMetadata, L extends KubernetesResourceList, B extends Builder<T>, D extends Doneable<T>>
  extends BaseResource<T, B, D> {

  private Class<L> listClazz;

  private Map<String, String> labels = new TreeMap<>();
  private Map<String, String> fields = new TreeMap<>();

  protected BaseResourceList(AsyncHttpClient httpClient, URL rootUrl, String resourceT, Class<T> clazz, Class<L> listClazz, Class<B> builderClazz, Class<D> updateableClazz) {
    super(httpClient, rootUrl, resourceT, clazz, builderClazz, updateableClazz);
    this.listClazz = listClazz;
  }

  protected Map<String, String> getFields() {
    return fields;
  }

  protected Map<String, String> getLabels() {
    return labels;
  }

  protected Class<L> getListClazz() {
    return listClazz;
  }

  public L list() throws KubernetesClientException {
    try {
      URL requestUrl = getNamespacedUrl();
      AsyncHttpClient.BoundRequestBuilder requestBuilder = getHttpClient().prepareGet(requestUrl.toString());
      if (labels != null && !labels.isEmpty()) {
        StringBuilder sb = new StringBuilder();
        for (Iterator<Map.Entry<String, String>> iter = labels.entrySet().iterator(); iter.hasNext(); ) {
          Map.Entry<String, String> entry = iter.next();
          sb.append(entry.getKey()).append("=").append(entry.getValue());
          if (iter.hasNext()) {
            sb.append(",");
          }
        }
        requestBuilder.addQueryParam("labelSelector", sb.toString());
      }
      if (fields != null && !fields.isEmpty()) {
        StringBuilder sb = new StringBuilder();
        for (Iterator<Map.Entry<String, String>> iter = fields.entrySet().iterator(); iter.hasNext(); ) {
          Map.Entry<String, String> entry = iter.next();
          sb.append(entry.getKey()).append("=").append(entry.getValue());
          if (iter.hasNext()) {
            sb.append(",");
          }
        }
        requestBuilder.addQueryParam("fieldSelector", sb.toString());
      }
      Future<Response> f = requestBuilder.execute();
      Response r = f.get();
      if (r.getStatusCode() != 200) {
        Status status = mapper.reader(Status.class).readValue(r.getResponseBodyAsStream());
        throw new KubernetesClientException(status.getMessage(), status.getCode(), status);
      }
      return mapper.reader(listClazz).readValue(r.getResponseBodyAsStream());
    } catch (InterruptedException | ExecutionException | IOException e) {
      throw KubernetesClientException.launderThrowable(e);
    }
  }

  public void delete() throws KubernetesClientException {
    try {
      L discoveredResources = list();

      for (Object resource : discoveredResources.getItems()) {
        // Dirty cast but should always be valid...
        HasMetadata metadataResource = (HasMetadata) resource;

        URL requestUrl = getRootUrl();
        if (metadataResource.getMetadata().getNamespace() != null) {
          requestUrl = new URL(requestUrl, "namespaces/" + metadataResource.getMetadata().getNamespace() + "/");
        }
        requestUrl = new URL(requestUrl, getResourceT() + "/" + metadataResource.getMetadata().getName());
        AsyncHttpClient.BoundRequestBuilder requestBuilder = getHttpClient().prepareDelete(requestUrl.toString());
        Future<Response> f = requestBuilder.execute();
        Response r = f.get();
        if (r.getStatusCode() != 200) {
          Status status = mapper.reader(Status.class).readValue(r.getResponseBodyAsStream());
          throw new KubernetesClientException(status.getMessage(), status.getCode(), status);
        }
      }
    } catch (InterruptedException | ExecutionException | IOException e) {
      throw KubernetesClientException.launderThrowable(e);
    }
  }

    public WebSocket watch(final Watcher<T> watcher) throws KubernetesClientException {
      try {
        URL requestUrl = getNamespacedUrl();
        AsyncHttpClient.BoundRequestBuilder requestBuilder = getHttpClient().prepareGet(requestUrl.toString().replaceFirst("^http", "ws"));
        if (labels != null && !labels.isEmpty()) {
          StringBuilder sb = new StringBuilder();
          for (Iterator<Map.Entry<String, String>> iter = labels.entrySet().iterator(); iter.hasNext(); ) {
            Map.Entry<String, String> entry = iter.next();
            sb.append(entry.getKey()).append("=").append(entry.getValue());
            if (iter.hasNext()) {
              sb.append(",");
            }
          }
          requestBuilder.addQueryParam("labelSelector", sb.toString());
        }
        if (fields != null && !fields.isEmpty()) {
          StringBuilder sb = new StringBuilder();
          for (Iterator<Map.Entry<String, String>> iter = fields.entrySet().iterator(); iter.hasNext(); ) {
            Map.Entry<String, String> entry = iter.next();
            sb.append(entry.getKey()).append("=").append(entry.getValue());
            if (iter.hasNext()) {
              sb.append(",");
            }
          }
          requestBuilder.addQueryParam("fieldSelector", sb.toString());
        }
        requestBuilder.addQueryParam("watch", "true");
        Future<WebSocket> f = requestBuilder.execute(new WebSocketUpgradeHandler.Builder().addWebSocketListener(
            new DefaultWebSocketListener() {

              private final Logger logger = LoggerFactory.getLogger(this.getClass());

              @Override
              public void onMessage(String message) {
                try {
                  WatchEvent event = mapper.reader(WatchEvent.class).readValue(message);
                  T obj = (T) event.getObject();
                  Watcher.Action action = Watcher.Action.valueOf(event.getType());
                  watcher.eventReceived(action, obj);
                } catch (IOException e) {
                  logger.error("Could not deserialize watch event: {}", message, e);
                } catch (ClassCastException e) {
                  logger.error("Received wrong type of object for watch", e);
                } catch (IllegalArgumentException e) {
                  logger.error("Invalid event type", e);
                }
              }
            }).build()
        );
        return f.get();
      } catch (MalformedURLException | InterruptedException | ExecutionException e) {
        throw KubernetesClientException.launderThrowable(e);
      }
    }

}

==> kubernetes-client-1.1.0-sources.jar: io/fabric8/kubernetes/client/internal/BaseResourceList.java <==
/**
 * Copyright (C) 2015 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.fabric8.kubernetes.client.internal;

import com.ning.http.client.AsyncHttpClient;
import com.ning.http.client.Response;
import com.ning.http.client.ws.DefaultWebSocketListener;
import com.ning.http.client.ws.WebSocket;
import com.ning.http.client.ws.WebSocketUpgradeHandler;
import io.fabric8.kubernetes.api.builder.Builder;
import io.fabric8.kubernetes.api.model.Doneable;
import io.fabric8.kubernetes.api.model.HasMetadata;
import io.fabric8.kubernetes.api.model.KubernetesResourceList;
import io.fabric8.kubernetes.api.model.Status;
import io.fabric8.kubernetes.api.model.WatchEvent;
import io.fabric8.kubernetes.client.KubernetesClientException;
import io.fabric8.kubernetes.client.Watcher;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

import static io.fabric8.kubernetes.client.internal.Utils.join;

public abstract class BaseResourceList<T extends HasMetadata, L extends KubernetesResourceList, B extends Builder<T>, D extends Doneable<T>>
  extends BaseResource<T, B, D> {

  private Class<L> listClazz;

  private Map<String, String> labels = new TreeMap<>();
  private Map<String, String> labelsNot = new TreeMap<>();
  private Map<String, String[]> labelsIn = new TreeMap<>();
  private Map<String, String[]> labelsNotIn = new TreeMap<>();
  private Map<String, String> fields = new TreeMap<>();

  protected BaseResourceList(AsyncHttpClient httpClient, URL rootUrl, String resourceT, Class<T> clazz, Class<L> listClazz, Class<B> builderClazz, Class<D> updateableClazz) {
    super(httpClient, rootUrl, resourceT, clazz, builderClazz, updateableClazz);
    this.listClazz = listClazz;
  }

  protected Map<String, String> getFields() {
    return fields;
  }

  protected Map<String, String> getLabels() {
    return labels;
  }

  public Map<String, String[]> getLabelsNotIn() {
    return labelsNotIn;
  }

  public Map<String, String[]> getLabelsIn() {
    return labelsIn;
  }

  public Map<String, String> getLabelsNot() {
    return labelsNot;
  }

  protected Class<L> getListClazz() {
    return listClazz;
  }

  public L list() throws KubernetesClientException {
    try {
      URL requestUrl = getNamespacedUrl();
      AsyncHttpClient.BoundRequestBuilder requestBuilder = getHttpClient().prepareGet(requestUrl.toString());
      StringBuilder sb = new StringBuilder();
      if (labels != null && !labels.isEmpty()) {
        if (sb.length() > 0) {
          sb.append(",");
        }
        for (Iterator<Map.Entry<String, String>> iter = labels.entrySet().iterator(); iter.hasNext(); ) {
          Map.Entry<String, String> entry = iter.next();
          sb.append(entry.getKey()).append("=").append(entry.getValue());
        }
      }
      if (labelsNot != null && !labelsNot.isEmpty()) {
        for (Iterator<Map.Entry<String, String>> iter = labelsNot.entrySet().iterator(); iter.hasNext(); ) {
          if (sb.length() > 0) {
            sb.append(",");
          }
          Map.Entry<String, String> entry = iter.next();
          sb.append(entry.getKey()).append("!=").append(entry.getValue());
        }
      }
      if (labelsIn != null && !labelsIn.isEmpty()) {
        for (Iterator<Map.Entry<String, String[]>> iter = labelsIn.entrySet().iterator(); iter.hasNext(); ) {
          if (sb.length() > 0) {
            sb.append(",");
          }
          Map.Entry<String, String[]> entry = iter.next();
          sb.append(entry.getKey()).append(" in ").append("(").append(join(entry.getValue())).append(")");
        }
      }
      if (labelsNotIn != null && !labelsNotIn.isEmpty()) {
        for (Iterator<Map.Entry<String, String[]>> iter = labelsNotIn.entrySet().iterator(); iter.hasNext(); ) {
          if (sb.length() > 0) {
            sb.append(",");
          }
          Map.Entry<String, String[]> entry = iter.next();
          sb.append(entry.getKey()).append(" notin ").append("(").append(join(entry.getValue())).append(")");
        }
      }
      if (sb.length() > 0) {
        requestBuilder.addQueryParam("labelSelector", sb.toString());
      }

      sb = new StringBuilder();
      if (fields != null && !fields.isEmpty()) {
        if (sb.length() > 0) {
          sb.append(",");
        }
        for (Iterator<Map.Entry<String, String>> iter = fields.entrySet().iterator(); iter.hasNext(); ) {
          Map.Entry<String, String> entry = iter.next();
          sb.append(entry.getKey()).append("=").append(entry.getValue());
        }
      }if (sb.length() > 0) {
        requestBuilder.addQueryParam("fieldSelector", sb.toString());
      }
      Future<Response> f = requestBuilder.execute();
      Response r = f.get();
      if (r.getStatusCode() != 200) {
        Status status = mapper.reader(Status.class).readValue(r.getResponseBodyAsStream());
        throw new KubernetesClientException(status.getMessage(), status.getCode(), status);
      }
      return mapper.reader(listClazz).readValue(r.getResponseBodyAsStream());
    } catch (InterruptedException | ExecutionException | IOException e) {
      throw KubernetesClientException.launderThrowable(e);
    }
  }

  public void delete() throws KubernetesClientException {
    try {
      L discoveredResources = list();

      for (Object resource : discoveredResources.getItems()) {
        // Dirty cast but should always be valid...
        HasMetadata metadataResource = (HasMetadata) resource;

        URL requestUrl = getRootUrl();
        if (metadataResource.getMetadata().getNamespace() != null) {
          requestUrl = new URL(requestUrl, "namespaces/" + metadataResource.getMetadata().getNamespace() + "/");
        }
        requestUrl = new URL(requestUrl, getResourceT() + "/" + metadataResource.getMetadata().getName());
        AsyncHttpClient.BoundRequestBuilder requestBuilder = getHttpClient().prepareDelete(requestUrl.toString());
        Future<Response> f = requestBuilder.execute();
        Response r = f.get();
        if (r.getStatusCode() != 200) {
          Status status = mapper.reader(Status.class).readValue(r.getResponseBodyAsStream());
          throw new KubernetesClientException(status.getMessage(), status.getCode(), status);
        }
      }
    } catch (InterruptedException | ExecutionException | IOException e) {
      throw KubernetesClientException.launderThrowable(e);
    }
  }

    public WebSocket watch(final Watcher<T> watcher) throws KubernetesClientException {
      try {
        URL requestUrl = getNamespacedUrl();
        AsyncHttpClient.BoundRequestBuilder requestBuilder = getHttpClient().prepareGet(requestUrl.toString().replaceFirst("^http", "ws"));
        if (labels != null && !labels.isEmpty()) {
          StringBuilder sb = new StringBuilder();
          for (Iterator<Map.Entry<String, String>> iter = labels.entrySet().iterator(); iter.hasNext(); ) {
            Map.Entry<String, String> entry = iter.next();
            sb.append(entry.getKey()).append("=").append(entry.getValue());
            if (iter.hasNext()) {
              sb.append(",");
            }
          }
          requestBuilder.addQueryParam("labelSelector", sb.toString());
        }
        if (fields != null && !fields.isEmpty()) {
          StringBuilder sb = new StringBuilder();
          for (Iterator<Map.Entry<String, String>> iter = fields.entrySet().iterator(); iter.hasNext(); ) {
            Map.Entry<String, String> entry = iter.next();
            sb.append(entry.getKey()).append("=").append(entry.getValue());
            if (iter.hasNext()) {
              sb.append(",");
            }
          }
          requestBuilder.addQueryParam("fieldSelector", sb.toString());
        }
        requestBuilder.addQueryParam("watch", "true");
        Future<WebSocket> f = requestBuilder.execute(new WebSocketUpgradeHandler.Builder().addWebSocketListener(
            new DefaultWebSocketListener() {

              private final Logger logger = LoggerFactory.getLogger(this.getClass());

              @Override
              public void onMessage(String message) {
                try {
                  WatchEvent event = mapper.reader(WatchEvent.class).readValue(message);
                  T obj = (T) event.getObject();
                  Watcher.Action action = Watcher.Action.valueOf(event.getType());
                  watcher.eventReceived(action, obj);
                } catch (IOException e) {
                  logger.error("Could not deserialize watch event: {}", message, e);
                } catch (ClassCastException e) {
                  logger.error("Received wrong type of object for watch", e);
                } catch (IllegalArgumentException e) {
                  logger.error("Invalid event type", e);
                }
              }
            }).build()
        );
        return f.get();
      } catch (MalformedURLException | InterruptedException | ExecutionException e) {
        throw KubernetesClientException.launderThrowable(e);
      }
    }

}
